import datetime

from osv import fields, osv

class account_invoice(osv.osv):
    _inherit = 'account.invoice'
#    
    _columns = {
        'create_date': fields.datetime('Date Created', readonly=True),
    }
    
    def sanity_check(self, cr, uid, context=None):
        self.sanity_check_fast(cr, uid, context)
        return 0

    def sanity_check_fast(self, cr, uid, context):
        ''' A set of fast sanity checks
        '''
        self.residual_when_paid(cr,uid,context)
        self.stuck_supplier_refunds(cr,uid,context)

    def residual_when_paid(self,cr,uid,context):
        case_id = self.pool.get('crm.helpdesk').sanity_test_create_stub_case(
            cr,
            uid,
            context,
            "Residual on a paid invoice",
            """\
This is a stub to track incidents where paid invoices still have a residual. 
""",
            'account_invoice_residual')

        invoice_ids = self.search(cr,
                                  uid,
                                  [('state', '=', 'paid'),
                                   ('residual', '!=', 0)])
        if not invoice_ids: 
            return # okay, no problems

        problem_writeup = "The following paid invoices have a residual:\n\n"
        for invoice in self.browse(cr, uid, invoice_ids):
            problem_writeup += (
                "%s (id %i) on %s for $%.2f with residual $%.2f to %s\n" % (
                    invoice.number,
                    invoice.id,
                    invoice.date_invoice,
                    invoice.amount_total,
                    invoice.residual,
                    invoice.partner_id.name))
        problem_writeup += "\n\n"

        # now that we've finished documenting the problem let's notify the 
        # appropriate users
        self.pool.get('crm.helpdesk').sanity_test_log(cr, 
                                                  uid,
                                                  context,
                                                  case_id,
                                                  problem_writeup)

    def stuck_supplier_refunds(self,cr,uid,context):
        case_id = self.pool.get('crm.helpdesk').sanity_test_create_stub_case(
            cr,
            uid,
            context,
            "Stuck supplier refunds",
            """\
This is a stub to track incidents where a supplier refund gets left in draft. 
""",
            'account_invoice_stuck_refund')

        now = datetime.datetime.now()
        stale_timestamp = now - datetime.timedelta(60)
        stale_date = stale_timestamp.strftime('%Y-%m-%d')
        
        # Find old supplier refunds, or supplier invoices with negative amount.
        invoice_ids = self.search(cr,
                                  uid,
                                  [('state', '=', 'draft'),
                                   ('create_date', '<', stale_date),
                                   '|',
                                   ('type', '=', 'in_refund'),
                                   '&',
                                   ('type', '=', 'in_invoice'),
                                   ('amount_total', '<', 0.0)])

        if not invoice_ids:
            return # okay, no problems
        
        invoice_types = dict(self._columns['type'].selection)

        problem_writeup = (
            "The following supplier refunds are stuck in draft:\n\n")
        for invoice in self.browse(cr, uid, invoice_ids):
            problem_writeup += (
                "%s (id %i) on %s for $%.2f to %s\n" % (
                    invoice_types[invoice.type],
                    invoice.id,
                    invoice.create_date,
                    invoice.amount_total,
                    invoice.partner_id.name))
#            problem_writeup += (
#                "%s (id %i) on %s for $%.2f with residual $%.2f to %s\n" % (
#                    invoice.number,
#                    invoice.id,
#                    invoice.date_invoice,
#                    invoice.amount_total,
#                    invoice.residual,
#                    invoice.partner_id.name))
        problem_writeup += "\n\n"

        # now that we've finished documenting the problem let's notify the 
        # appropriate users
        self.pool.get('crm.helpdesk').sanity_test_log(cr, 
                                                  uid,
                                                  context,
                                                  case_id,
                                                  problem_writeup)

account_invoice()
    
